#include	"compiler.h"
#include	<ddraw.h>
#include	"dd2.h"


WORD dd2_get16pal(DD2_T *dd2, DWORD pal) {

	RGB32	ret;

	ret.d = pal & dd2->pal16.d;
	return((((WORD)ret.p.g) << dd2->l16g) |
			(((WORD)ret.p.r) << dd2->l16r) | (ret.p.b >> dd2->r16b));
}


BOOL dd2_create(HWND hwnd, DD2_T *dd2, DWORD width, DWORD height) {

	DDSURFACEDESC	ddsd;

	if (!dd2) {
		return(FAILURE);
	}
	ZeroMemory(dd2, sizeof(DD2_T));

	if (DirectDrawCreate(NULL, &dd2->ddraw1, NULL) != DD_OK) {
		return(FAILURE);
	}
	dd2->ddraw1->QueryInterface(IID_IDirectDraw2, (void **)&dd2->ddraw);

	dd2->ddraw->SetCooperativeLevel(hwnd, DDSCL_NORMAL);

	ZeroMemory(&ddsd, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	ddsd.dwFlags = DDSD_CAPS;
	ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;

	if (dd2->ddraw->CreateSurface(&ddsd, &dd2->primsurf, NULL) != DD_OK) {
		return(FAILURE);
	}

	dd2->ddraw->CreateClipper(0, &dd2->clipper, NULL);
	dd2->clipper->SetHWnd(0, hwnd);
	dd2->primsurf->SetClipper(dd2->clipper);

	ZeroMemory(&dd2->ddpf, sizeof(DDPIXELFORMAT));
	dd2->ddpf.dwSize = sizeof(DDPIXELFORMAT);
	if (dd2->primsurf->GetPixelFormat(&dd2->ddpf) != DD_OK) {
		return(FAILURE);
	}

	ZeroMemory(&ddsd, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	ddsd.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;
	ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
	ddsd.dwWidth = width;
	ddsd.dwHeight = height;

	if (dd2->ddraw->CreateSurface(&ddsd, &dd2->backsurf, NULL) != DD_OK) {
		return(FAILURE);
	}
	if (dd2->ddpf.dwRGBBitCount == 8) {
		HDC 	hdc;

		hdc = GetDC(hwnd);
		GetSystemPaletteEntries(hdc, 0, 256, dd2->pal);
		ReleaseDC(hwnd, hdc);
		dd2->ddraw->CreatePalette(DDPCAPS_8BIT, dd2->pal, &dd2->palette, 0);
		dd2->primsurf->SetPalette(dd2->palette);
	}
	else if (dd2->ddpf.dwRGBBitCount == 16) {
		WORD	bit;
		BYTE	cnt;

		dd2->pal16.d = 0;
		for (bit=1; (bit) && (!(dd2->ddpf.dwBBitMask & bit)); bit<<=1);
		for (dd2->r16b=8; (dd2->r16b) && (dd2->ddpf.dwBBitMask & bit);
													dd2->r16b--, bit<<=1) {
			dd2->pal16.p.b >>= 1;
			dd2->pal16.p.b |= 0x80;
		}
		for (dd2->l16r=0, bit=1; (bit) && (!(dd2->ddpf.dwRBitMask & bit));
													dd2->l16r++, bit<<=1);
		for (cnt=0x80; (cnt) && (dd2->ddpf.dwRBitMask & bit);
													cnt>>=1, bit<<=1) {
			dd2->pal16.p.r |= cnt;
		}
		for (; cnt; cnt>>=1) dd2->l16r--;
		for (dd2->l16g=0, bit=1; (bit) && (!(dd2->ddpf.dwGBitMask & bit));
													dd2->l16g++, bit<<=1);
		for (cnt=0x80; (cnt) && (dd2->ddpf.dwGBitMask & bit);
													cnt>>=1, bit<<=1) {
			dd2->pal16.p.g |= cnt;
		}
		for (; cnt; cnt>>=1) dd2->l16g--;
	}
	else if (dd2->ddpf.dwRGBBitCount == 24) {
	}
	else if (dd2->ddpf.dwRGBBitCount == 32) {
	}
	else {
		return(FAILURE);
	}
	dd2->vram.bit = (BYTE)dd2->ddpf.dwRGBBitCount;
	dd2->cliping = 0;
	return(SUCCESS);
}


void dd2_release(DD2_T *dd2) {

	if (dd2) {
		RELEASE(dd2->backsurf);
		RELEASE(dd2->palette);
		RELEASE(dd2->clipper);
		RELEASE(dd2->primsurf);
		RELEASE(dd2->ddraw);
		RELEASE(dd2->ddraw1);
	}
}

// ------

BOOL dd2_bsurflock(DD2_T *dd2) {

	if ((dd2) && (dd2->ddraw1)) {
		DDSURFACEDESC	surface;

		ZeroMemory(&surface, sizeof(DDSURFACEDESC));
		surface.dwSize = sizeof(DDSURFACEDESC);

		if (dd2->backsurf->Lock(NULL, &surface,
										DDLOCK_WAIT, NULL) == DD_OK) {
			dd2->vram.ptr = (BYTE *)surface.lpSurface;
			dd2->vram.align = surface.lPitch;
			return(SUCCESS);
		}
	}
	return(FAILURE);
}


void dd2_bsurfunlock(DD2_T *dd2) {

	if ((dd2) && (dd2->ddraw1)) {
		dd2->backsurf->Unlock(NULL);
	}
}


void dd2_blt(HWND hwnd, DD2_T *dd2, const POINT *pt, const RECT *rect) {

	POINT	clipt;
	RECT	scrn;

	if ((dd2) && (dd2->ddraw1)) {
		clipt = *pt;
		ClientToScreen(hwnd, &clipt);
		scrn.left = clipt.x;
		scrn.top = clipt.y;
		scrn.right = clipt.x + rect->right - rect->left;
		scrn.bottom = clipt.y + rect->bottom - rect->top;
		if (dd2->primsurf->Blt(&scrn, dd2->backsurf, (RECT *)rect,
									DDBLT_WAIT, NULL) == DDERR_SURFACELOST) {
			dd2->primsurf->Restore();
			dd2->backsurf->Restore();
		}
	}
}

